/*!
 * \file      app_drv_ch57x_utc_time.c
 *
 * \brief     ch57x utc time implementation(CH579/578/577 only)
 *
 * \copyright Revised BSD License, see section \ref LICENSE.
 *
 * \code
 *
 *              (C)2002-2020 wch
 *
 * \endcode
 *
 * \author    kingsley zhang ( @wch )
 *
 */






#include "CH57x_common.h"
#include "app_drv_ch57x_utc_time.h"


//return local time in seconds
uint32_t app_drv_ch57x_utc_time_get_timestamp(void) {
    uint32_t  t;
    uint16_t  day, sec2, t32k;

    day = R32_RTC_CNT_DAY & 0x3FFF;
    sec2 = R16_RTC_CNT_2S;
    t32k = R16_RTC_CNT_32K;
    //PRINT("day=%d\r\n",day);
    t = day*86400+(sec2<<1) + ((t32k<0x8000)?0:1);		//
    return t;
}

//set local rtc cnt in seconds
//the max timestamp in hardware is (0x3fff*86400)+(86400-1) about 44 year
//when use this in ble, TMOS_TimerInit must be executed after it.
//and this will cause ble connection lost
void app_drv_ch57x_utc_time_set_timestamp(uint32_t timestamp) {
    uint16_t days;
    uint32_t seconds;

    days = timestamp/86400;                 //for use in days register
    seconds = timestamp%86400;              //the left seconds less than one days
    seconds >>= 1;                          //we use seconds in 2s in the register
    seconds <<= 16;                         //the higher 16 bits store the seconds in 2s
    seconds |= (timestamp&1)?(0x8000):(0);  //get the last bit of seconds

    //PRINT("timestamp=%d\r\n",timestamp);
    //PRINT("days=%d,seconds=%d\r\n",days,seconds);
    R8_SAFE_ACCESS_SIG = SAFE_ACCESS_SIG1;		// 밲ȫģʽ
    R8_SAFE_ACCESS_SIG = SAFE_ACCESS_SIG2;

    R32_RTC_TRIG = days;
    R8_RTC_MODE_CTRL |= RB_RTC_LOAD_HI;
    R32_RTC_TRIG = seconds;
    R8_RTC_MODE_CTRL |= RB_RTC_LOAD_LO;

    R8_SAFE_ACCESS_SIG = 0;
}


